#version 430 core

in vec2 f_uv;
in vec3 f_object_normal;
in vec3 f_object_to_camera_direction;
in vec3 f_object_to_light_direction[8];
in float f_attenuation[8];

layout(location = 0) out vec4 o_color;

struct Light {
	vec3 world_light_position;
	vec3 light_color_ambient;
	vec3 light_color_diffuse;
	vec3 light_color_specular;
	float attenuation_constant;
	float attenuation_linear;
	float attenuation_quadratic;
	float shininess;
};

layout(location = 4) uniform sampler2D texture_diffuse;
layout(location = 5) uniform sampler2D texture_specular;
layout(location = 6) uniform vec3 u_camera_position;
layout(location = 7) uniform int instructions;
layout(location = 8) uniform Light u_lights[8]; // TODO: are 8 light sources enough?!

// TODO: Set/get constants somewhere else
float ka = 0.2f;
float kd = 0.7f;
float ks = 0.6f;

int RENDER_TEXTURE_ONLY = 0x0001;

void main()
{
	// Get material colors
	vec3 material_diffuse_color = texture(texture_diffuse, f_uv).rgb;
	vec3 material_specular_color = clamp(2.0 * texture(texture_diffuse, f_uv).rgb, vec3(0.0, 0.0, 0.0), vec3(1.0, 1.0, 1.0));// TODO: texture(texture_specular, f_uv).rgb;
	vec3 material_ambient_color = material_diffuse_color   * vec3(1.0, 0.97, 0.85);

	if( uint(instructions & RENDER_TEXTURE_ONLY) == RENDER_TEXTURE_ONLY){
		o_color = vec4(material_diffuse_color, 1.0);
	}
	else{
		vec3 out_color_rgb = vec3(0.0, 0.0, 0.0);
		for(int i = 0; i < 2; i++)  // TODO: hardcode #lights?!
		{
			// Normalize the vectors or strange things will happen
			vec3 n = normalize(f_object_normal);
			vec3 l = normalize(f_object_to_light_direction[i]);
			vec3 v = normalize(f_object_to_camera_direction);

			// Calculating the reflection vector
			vec3 r = reflect(-l, n);

			// Calculation cos between normal and light | eye and reflection
			float cos_theta = clamp(dot(n, l), 0.0, 1.0);
			float cos_alpha = clamp(dot(v, r), 0.0, 1.0);

			// Do the shading
			vec3 color = kd * material_diffuse_color * u_lights[i].light_color_diffuse * cos_theta +
						 ks * material_specular_color * u_lights[i].light_color_specular * pow(cos_alpha, u_lights[i].shininess);

			// Apply light attenuation
			color *= f_attenuation[i];

			// Apply ambient light here because it should not be affected by light attenuation
			color += ka * material_ambient_color * u_lights[i].light_color_ambient;

			// Accumulate the color
			out_color_rgb += color;
		}

		// Handle spillover
		vec3 spillover = max(vec3(0.0, 0.0, 0.0), out_color_rgb - vec3(1.0, 1.0, 1.0));
		out_color_rgb.r += spillover.g / 2.0 + spillover.b / 2.0;
		out_color_rgb.g += spillover.r / 2.0 + spillover.b / 2.0;
		out_color_rgb.b += spillover.r / 2.0 + spillover.g / 2.0;
	
		o_color = vec4(out_color_rgb, 1.0);
	}
}